from setuptools import setup, find_packages

setup(
    name='pytezergis',
    version='1.2',
    packages=find_packages(),
    description='Pacote responsável pela manipulação de dados geospaciais',
    author='Manoel Neco Jr',
    author_email='necoengenheiro@gmail.com',
)
